// Update Notifier
// By Todd Long <longfocus@gmail.com>
// http://www.longfocus.com/firefox/updatenotifier/

const UN_CC = Components.classes;
const UN_CI = Components.interfaces;

const UN_MOZILLA_ITEM = "urn:mozilla:item:";
const UN_DEFAULT_THEME = "classic/1.0";

const UN_ITEM_UPDATE_URL = "availableUpdateURL";
const UN_ITEM_UPDATE_VERSION = "availableUpdateVersion";
const UN_ITEM_UPDATE_INTERNAL_NAME = "internalName";

const UN_NOTIFY_TOPIC = "UN:update-topic";
const UN_NOTIFY_TYPE_CHANGE = "UN:update-type-change";
const UN_NOTIFY_TYPE_RESTART = "UN:update-type-restart";
const UN_NOTIFY_TYPE_BUSY_NONE = "UN:update-type-busy-none";
const UN_NOTIFY_TYPE_BUSY_CHECKING = "UN:update-type-busy-checking";
const UN_NOTIFY_TYPE_BUSY_DOWNLOAD = "UN:update-type-busy-download";
const UN_NOTIFY_TYPE_BUSY_INSTALL = "UN:update-type-busy-install";

const UN_UPDATE_ADD = "update-add";
const UN_UPDATE_REMOVE = "update-remove";

const UN_TYPE_EXTENSION = UN_CI.nsIUpdateItem.TYPE_EXTENSION;
const UN_TYPE_THEME = UN_CI.nsIUpdateItem.TYPE_THEME;

function unManager() {}
unManager.prototype = {
  _items: new Array(),
  _firstTime: true,
  _needRestart: false,
  _busyStatus: UN_NOTIFY_TYPE_BUSY_NONE,
  _theme: UN_DEFAULT_THEME,
  _themeId: null,
  
  get restart() { return this._needRestart; },
  get status() { return this._busyStatus; },
  
  load: function(aCheck, aTheme)
  {
    if (this._firstTime)
    {
      this._firstTime = false;
      this._theme = aTheme;
      
      this._em = UN_CC["@mozilla.org/extensions/manager;1"].getService(UN_CI.nsIExtensionManager);
      this._cs = UN_CC['@mozilla.org/consoleservice;1'].getService(UN_CI.nsIConsoleService);
      this._observer = UN_CC["@mozilla.org/observer-service;1"].getService(UN_CI.nsIObserverService);
      
      // Adds observers for item updates
      this._em.datasource.AddObserver(this);
      this._em.addUpdateListener(this);
      
      if (aCheck)
        this.checkUpdates();
    }
    
    // Checks the RDF for updates
    this._loadItemUpdates();
  },
  
  getNumItems: function()
  {
    return this._items ? this._items.length : 0;
  },
  
  getNumItemsByType: function(aType)
  {
    var num = 0;
    
    for (var i = 0; i < this._items.length; i++)
      if (this._items[i].type == aType)
        num++;
    
    return num;
  },
  
  getItemByIndex: function(aNum)
  {
    return this._items ? this._items[aNum] : null;
  },
  
  checkUpdates: function()
  {
    var items = this._em.getItemList(UN_CI.nsIUpdateItem.TYPE_ANY, {});
    this._em.update(items, items.length, false, this);
  },
  
  installUpdates: function()
  {
    var itemList = new Array();
    
    for (var i = 0; i < this._items.length; i++)
      itemList.push(this._em.getItemForID(this._items[i].id));
    
    if (itemList.length > 0)
      this._em.addDownloads(itemList, itemList.length, true);
  },
  
  _loadItemUpdates: function()
  {
    var res = this._em.datasource.GetAllResources();
    
    while (res.hasMoreElements())
    {
      var element = res.getNext().QueryInterface(UN_CI.nsIRDFResource);
      this._itemUpdateAction(this._em.datasource, element, UN_UPDATE_ADD);
    }
  },
  
  _itemUpdateAction: function(aDataSource, aSource, aAction)
  {
    var arcLabelsOut = aDataSource.ArcLabelsOut(aSource);
    var curItem = this._em.getItemForID(aSource.Value.replace(UN_MOZILLA_ITEM, ""));
    var item = new unItem(); //UN_CC["@longfocus.com/updatenotifier/item;1"].createInstance(UN_CI.unIItem);
    
    item.id = curItem.id;
    item.name = curItem.name;
    item.updateURL = null;
    item.curVersion = curItem.version;
    item.newVersion = null;
    
    switch (curItem.type)
    {
      case UN_TYPE_EXTENSION:
        item.type = "extension";
        break;
      case UN_TYPE_THEME:
        item.type = "theme";
        break;
      default: // Don't know, don't care
        return;
    }
    
    while(arcLabelsOut.hasMoreElements())
    {
      labelOut = arcLabelsOut.getNext().QueryInterface(UN_CI.nsIRDFResource);
      
      var target = aDataSource.GetTarget(aSource, labelOut, true);
      var rdfLiteral = null;
      
      if (target instanceof UN_CI.nsIRDFLiteral)
        rdfLiteral = target.Value;
      
      if (labelOut.Value.indexOf(UN_ITEM_UPDATE_URL) > -1)
        item.updateURL = rdfLiteral;
      else if (labelOut.Value.indexOf(UN_ITEM_UPDATE_VERSION) > -1)
        item.newVersion = rdfLiteral;
      else if (labelOut.Value.indexOf(UN_ITEM_UPDATE_INTERNAL_NAME) > -1) {
        // Checks if the theme matches
        if (rdfLiteral == this._theme)
          this._themeId = item.id;
      }
    }
    
    switch (aAction)
    {
      case UN_UPDATE_ADD:
        if (item.updateURL != null && item.newVersion != null)
          this._addItemUpdate(item);
        break;
      case UN_UPDATE_REMOVE:
        this._removeItemUpdate(item);
        break;
    }
  },
  
  _addItemUpdate: function(aItem)
  {
    // Gets the item's index
    var index = this._getItemIndex(aItem);
    
    if (index > -1) {
      // Checks if item details match
      if (this._items[index].id == aItem.id &&
          this._items[index].name == aItem.name &&
          this._items[index].updateURL == aItem.updateURL &&
          this._items[index].curVersion == aItem.curVersion &&
          this._items[index].newVersion == aItem.newVersion) return;
      
      this._items[index] = aItem;
    }
    else
       this._items[this._items.length] = aItem;
    
    // Notifies about the changes
    this._observer.notifyObservers(null, UN_NOTIFY_TOPIC, UN_NOTIFY_TYPE_CHANGE);
  },
  
  _removeItemUpdate: function(aItem)
  {
    // Gets the item's index
    var index = this._getItemIndex(aItem);
    
    // Checks for valid index
    if (index > -1)
    {
      // Removes the item
      this._items.splice(index, 1);
      
      // Notifies about the changes
      this._observer.notifyObservers(null, UN_NOTIFY_TOPIC, UN_NOTIFY_TYPE_CHANGE);
    }
  },
  
  _getItemIndex: function(aItem)
  {
    var index = -1;
    
    for (var i = 0; i < this._items.length && index == -1; i++)
      if (this._items[i].id == aItem.id)
        index = i;
    
    return index;
  },
  
  _setBusyStatus: function(aStatus)
  {
    // Sets the status
    this._busyStatus = aStatus;
    
    // Notifies the status
    this._observer.notifyObservers(null, UN_NOTIFY_TOPIC, aStatus);
  },
  
  /**
   * nsIRDFObserver
   */
  onChange: function(aDataSource, aSource, aProperty, aOldTarget, aNewTarget)
  {
    var pv = aProperty.QueryInterface(UN_CI.nsIRDFResource).Value;
    
    if (pv.indexOf(UN_ITEM_UPDATE_URL) > -1 ||
        pv.indexOf(UN_ITEM_UPDATE_VERSION) > -1) {
      this._itemUpdateAction(aDataSource, aSource, UN_UPDATE_ADD);
    }
  },
  
  onUnassert: function(aDataSource, aSource, aProperty, aTarget)
  {
    var pv = aProperty.QueryInterface(UN_CI.nsIRDFResource).Value;
    
    if (pv.indexOf(UN_ITEM_UPDATE_URL) > -1 ||
        pv.indexOf(UN_ITEM_UPDATE_VERSION) > -1) {
      this._itemUpdateAction(aDataSource, aSource, UN_UPDATE_REMOVE);
    }
  },
  
  onAssert: function(aDataSource, aSource, aProperty, aTarget) {},
  onBeginUpdateBatch: function(aDataSource) {},
  onEndUpdateBatch: function(aDataSource) {},
  onMove: function(aDataSource, aOldSource, aNewSource, aProperty, aTarget) {},
  
  /**
   * nsIAddonUpdateCheckListener
   */
  onUpdateStarted: function()
  {
    // Notify checking status
    this._setBusyStatus(UN_NOTIFY_TYPE_BUSY_CHECKING);
  },
  
  onUpdateEnded: function()
  {
    // Notify none status
    this._setBusyStatus(UN_NOTIFY_TYPE_BUSY_NONE);
  },
  
  onAddonUpdateEnded: function(aAddon, aStatus) {},
  onAddonUpdateStarted: function(aAddon) {},
  
  /**
   * nsIAddonUpdateListener
   */
  onStateChange: function (aAddon, aState, aValue)
  {
    //this._cs.logStringMessage(aState + " - " + aValue);
    
    // 0, 1, 2, 3, 4
    // 0, 0, 0, -207, 0
    
    // Checks for error...not really needed
    if (aValue != 0 && aValue != 999) {
      // Removes the dowload from the list
      this._em.removeDownload(aAddon.xpiURL);
    }
    
    switch (aState)
    {
      case UN_CI.nsIXPIProgressDialog.DOWNLOAD_START:
      {
        // Notify of update download
        this._setBusyStatus(UN_NOTIFY_TYPE_BUSY_DOWNLOAD);
        break;
      }
      case UN_CI.nsIXPIProgressDialog.INSTALL_START:
      {
        // Notify of update install
        this._setBusyStatus(UN_NOTIFY_TYPE_BUSY_INSTALL);
        break;
      }
      case UN_CI.nsIXPIProgressDialog.DOWNLOAD_DONE:
      case UN_CI.nsIXPIProgressDialog.INSTALL_DONE:
      {
        if ((aValue == 0 || aValue == 999) && aState == 3 &&
            (aAddon.type == UN_TYPE_EXTENSION || aAddon.id == this._themeId))
          this._needRestart = true;
        
        // Notify update not busy
        this._setBusyStatus(UN_NOTIFY_TYPE_BUSY_NONE);
        break;
      }
      case UN_CI.nsIXPIProgressDialog.DIALOG_CLOSE:
      {
        // Check for restart
        if (this._needRestart)
          // Notify restart is required
          this._observer.notifyObservers(null,
                           UN_NOTIFY_TOPIC, UN_NOTIFY_TYPE_RESTART);
        
        break;
      }
    }
  },
  
  onProgress: function (aAddon, aValue, aMaxValue) {},
  
  QueryInterface: function(iid)
  {
    if (!iid.equals(UN_CI.unIManager) &&
        !iid.equals(UN_CI.nsIAddonUpdateCheckListener) &&
        !iid.equals(UN_CI.nsIAddonUpdateListener) &&
        !iid.equals(UN_CI.nsISupports))
      throw Components.results.NS_ERROR_NO_INTERFACE;
    return this;
  }
}

function unItem() {}
unItem.prototype = {
  _id: null,
  _name: null,
  _type: null,
  _updateURL: null,
  _curVersion: null,
  _newVersion: null,
  
  get id() { return this._id; },
  get name() { return this._name; },
  get type() { return this._type; },
  get updateURL() { return this._updateURL; },
  get curVersion() { return this._curVersion; },
  get newVersion() { return this._newVersion; },
  
  set id(aId) { this._id = aId; },
  set name(aName) { this._name = aName; },
  set type(aType) { this._type = aType; },
  set updateURL(aURL) { this._updateURL = aURL; },
  set curVersion(aVersion) { this._curVersion = aVersion; },
  set newVersion(aVersion) { this._newVersion = aVersion; }
}

var myModule = {
  firstTime: true,
  
  myCID: Components.ID("{0090c2b0-9e45-11da-a746-0800200c9a66}"),
  myDesc: "Manager for addon updates",
  myProgID: "@longfocus.com/updatenotifier/manager;1",
  myFactory: {
    createInstance: function (outer, iid) {
      if (outer != null)
        throw Components.results.NS_ERROR_NO_AGGREGATION;
      
      return (new unManager()).QueryInterface(iid);
    }
  },

  registerSelf: function (compMgr, fileSpec, location, type)
  {
    if (this.firstTime) {
      this.firstTime = false;
      throw Components.results.NS_ERROR_FACTORY_REGISTER_AGAIN;
    }
    
    compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
    compMgr.registerFactoryLocation(this.myCID, this.myDesc, this.myProgID, fileSpec, location, type);
  },
 
  getClassObject: function (compMgr, cid, iid)
  {
    if (!cid.equals(this.myCID))
      throw Components.results.NS_ERROR_NO_INTERFACE;
    
    if (!iid.equals(Components.interfaces.nsIFactory))
      throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    
    return this.myFactory;
  },
  
  canUnload: function(compMgr) { return true; }
};

function NSGetModule(compMgr, fileSpec) { return myModule; }
