// Update Notifier
// By Todd Long <longfocus@gmail.com>
// http://www.longfocus.com/firefox/updatenotifier/

const UN_NOTIFY_TOPIC = "UN:update-topic";
const UN_NOTIFY_TYPE_CHANGE = "UN:update-type-change";
const UN_NOTIFY_TYPE_RESTART = "UN:update-type-restart";
const UN_NOTIFY_TYPE_BUSY_NONE = "UN:update-type-busy-none";
const UN_NOTIFY_TYPE_BUSY_CHECKING = "UN:update-type-busy-checking";
const UN_NOTIFY_TYPE_BUSY_DOWNLOAD = "UN:update-type-busy-download";
const UN_NOTIFY_TYPE_BUSY_INSTALL = "UN:update-type-busy-install";

function UN_onOverlayLoad() {
  UN_gOverlay.load();
}

function UN_onOverlayUnload() {
  UN_gOverlay.unload();
}

var UN_gOverlay = {
  _em: null,
  _branchWatch: null,
  _observer: null,
  _updates: null,
  _restart: false,
  _busyStatus: UN_NOTIFY_TYPE_BUSY_NONE,
  
  load: function()
  {
    // Initialization
    this._em = UN_CC["@mozilla.org/extensions/manager;1"].getService(UN_CI.nsIExtensionManager);
    
    // Preferences
    this._branchWatch = UN_getBranch().QueryInterface(UN_CI.nsIPrefBranch2);
    this._branchWatch.addObserver("", this, false);
    
    // Adds the observer for new updates
    this._observer = UN_CC["@mozilla.org/observer-service;1"].getService(UN_CI.nsIObserverService);
    this._observer.addObserver(this, UN_NOTIFY_TOPIC, false);
    
    // Loads the update manager
    this._updates = UN_CC["@longfocus.com/updatenotifier/manager;1"].getService(UN_CI.unIManager);
    this._updates.load(UN_getBoolPref("startup.check"), UN_getCurrentTheme());
    
    // Initialize update state
    this._restart = this._updates.restart;
    this._busyStatus = this._updates.status;
    
    // Checks the preferences
    this._checkPrefs();
  },
  
  unload: function()
  {
    // Removes the observers
    this._branchWatch.removeObserver("", this);
    this._observer.removeObserver(this, UN_NOTIFY_TOPIC);
  },
  
  _checkPrefs: function()
  {
    // Check first time run
    if (UN_getBoolPref("first-time"))
    {
      if (!UN_getEBI("un-toolbarbutton"))
      {
        var tb = UN_getEBI("toolbar-menubar");
        
        if (!tb)
          tb = UN_getEBI("mail-bar");
        
        if (UN_getEBI("throbber-box"))
          tb.currentSet = tb.currentSet.replace("throbber-box", "un-toolbarbutton,throbber-box");
        else
          tb.currentSet += ",un-toolbarbutton";
        
        tb.setAttribute("currentset", tb.currentSet);
        document.persist(tb.id, "currentset");
        
        if (UN_isFirefox()) {
          try {
            BrowserToolboxCustomizeDone(true);
          } catch (e) {}
        }
      }
      
      // Set first time run to false
      UN_setBoolPref("first-time", "false");
    }
    
    // Updates the icons
    this._updateIcons();
    
    // Checks the statusbar
    this._checkStatusbar();
  },
  
  _updateIcons: function()
  {
    var extensions = (this._updates.getNumItemsByType("extension") > 0);
    var themes = (this._updates.getNumItemsByType("theme") > 0);
    var none = UN_getBoolPref("icon.always-display");
    var collapsed = (!extensions && !themes && !none && !this._restart);
    
    // Initializes the icon
    var icon = "none";
    
    // Sets the appropriate icon
    if (this._busyStatus != UN_NOTIFY_TYPE_BUSY_NONE)
      icon = "busy";
    else if (this._restart)
      icon = "restart"
    else if (extensions && !themes)
      icon = "extension"
    else if (themes && !extensions)
      icon = "theme";
    else if (extensions && themes)
      icon = "both";
    
    // Checks if toolbar button is visible
    if (UN_getEBI("un-toolbarbutton")) {
  	  // Sets the toolbar attributes
  	  UN_setEBIA("un-toolbarbutton", "icon", icon);
      UN_setEBIA("un-toolbarbutton", "collapsed", collapsed);
    }
    
    // Sets the statusbar attributes
    UN_setEBIA("un-statusbar-panel", "icon", icon);
    UN_setEBIA("un-statusbar-panel", "collapsed", collapsed);
  },
  
  _checkStatusbar: function()
  {
    var sbar = UN_getEBI("status-bar");
    var sbarPanel = UN_getEBI("un-statusbar-panel");
    
    sbarPanel.hidden = !UN_getBoolPref("statusbar");
    sbar.removeChild(sbarPanel);
    
    if (UN_getBoolPref("statusbar.always-last"))
      sbar.appendChild(sbarPanel);
    else {
      var pos = UN_getIntPref("statusbar.position");
      sbar.insertBefore(sbarPanel, sbar.childNodes[pos]);
    }
  },
  
  checkUpdates: function()
  {
    // Check updates
    this._updates.checkUpdates();
  },
  
  installUpdates: function()
  {
    // Installs updates
    this._updates.installUpdates();
  },
  
  checkRestart: function(aAuto)
  {
    var msg = null;
    var display = true;
    
    if (aAuto)
      msg = UN_getBundleFString("un-confirm-restart-auto", [UN_getPlatform()]);
    else
      msg = UN_getBundleFString("un-confirm-restart", [UN_getPlatform()]);
    
    try {
      if (getBrowser() != UN_getBrowser().getBrowser())
        display = false;
    } catch(e) {}
    
    // Check for restart
    if (display && UN_confirm(msg) == true)
      UN_restart();
  },
  
  openType: function(aType)
  {
    if (UN_isFirefox())
      BrowserOpenExtensions(aType);
    else
      openExtensions(aType);
  },
  
  populatePopup: function(aId)
  {
    var popup = UN_getEBI(aId);
    var item = null;
    
    while (popup.hasChildNodes())
      popup.removeChild(popup.lastChild);
    
    item = document.createElement("menuitem");
    item.setAttribute("label", UN_getBundleString("tb-popup-menu-check-all-updates"));
    item.setAttribute("oncommand", "UN_gOverlay.checkUpdates();");
    popup.appendChild(item);
    
    item = document.createElement("menuitem");
    item.setAttribute("label", UN_getBundleString("tb-popup-menu-install-all-updates"));
    item.setAttribute("oncommand", "UN_gOverlay.installUpdates();");
    item.setAttribute("disabled", (this._updates.getNumItems() == 0));
    popup.appendChild(item);
    
    item = document.createElement("menuitem");
    item.setAttribute("label", UN_getBundleFString("tb-popup-menu-restart", [UN_getPlatform()]));
    item.setAttribute("oncommand", "UN_gOverlay.checkRestart(false);");
    //item.setAttribute("disabled", !this._restart);
    popup.appendChild(item);
    
    popup.appendChild(document.createElement("menuseparator"));
    
    item = document.createElement("menuitem");
    item.setAttribute("label", UN_getBundleString("tb-popup-menu-extensions"));
    item.setAttribute("oncommand", "UN_gOverlay.openType('extensions');");
    popup.appendChild(item);
    
    item = document.createElement("menuitem");
    item.setAttribute("label", UN_getBundleString("tb-popup-menu-themes"));
    item.setAttribute("oncommand", "UN_gOverlay.openType('themes');");
    popup.appendChild(item);
    
    popup.appendChild(document.createElement("menuseparator"));
    
    item = document.createElement("menuitem");
    item.setAttribute("label", UN_getBundleString("tb-popup-menu-homepage"));
    item.setAttribute("oncommand", "UN_visitSite(UN_WEBSITE);");
    popup.appendChild(item);
    
    item = document.createElement("menuitem");
    item.setAttribute("label", UN_getBundleString("tb-popup-menu-options"));
    item.setAttribute("default", "true");
    item.setAttribute("oncommand", "window.openDialog('chrome://updatenotifier/content/options.xul', 'UN:Prefs', 'centerscreen,chrome,dependent');");
    popup.appendChild(item);
  },
  
  populateTooltip: function()
  {
    var numUpdates = this._updates.getNumItems();
    var updates = (this._checking || this._restart || numUpdates == 0);
    
    // Set tooltip
    if (this._busyStatus == UN_NOTIFY_TYPE_BUSY_CHECKING)
    {
      // Checking for updates
      UN_setEBIA("un-toolbar-tooltip-header-label", "value", UN_getBundleString("tb-tooltip-checking"));
    }
    else if (this._busyStatus == UN_NOTIFY_TYPE_BUSY_DOWNLOAD)
    {
      // Installing new updates
      UN_setEBIA("un-toolbar-tooltip-header-label", "value", UN_getBundleString("tb-tooltip-download"));
    }
    else if (this._busyStatus == UN_NOTIFY_TYPE_BUSY_INSTALL)
    {
      // Installing new updates
      UN_setEBIA("un-toolbar-tooltip-header-label", "value", UN_getBundleString("tb-tooltip-install"));
    }
    else if (this._restart)
    {
      // Restart is required
      UN_setEBIA("un-toolbar-tooltip-header-label", "value", UN_getBundleString("tb-tooltip-restart"));
    }
    else if (numUpdates == 0)
    {
      // No available updates
      UN_setEBIA("un-toolbar-tooltip-header-label", "value", UN_getBundleString("tb-tooltip-no-updates"));
    }
    else
    {
      var tte = UN_getEBI("un-toolbar-tooltip-updates-extensions");
      var ttt = UN_getEBI("un-toolbar-tooltip-updates-themes");
      
      // How many updates
      if (numUpdates == 1)
      {
        // One new update is available
        UN_setEBIA("un-toolbar-tooltip-header-label", "value", UN_getBundleString("tb-tooltip-new-update"));
      }
      else
      {
        // Multiple updates are available
        UN_setEBIA("un-toolbar-tooltip-header-label", "value", UN_getBundleFString("tb-tooltip-new-updates", [numUpdates]));
      }
      
      // Remove all extensions
      while (tte.hasChildNodes())
        tte.removeChild(tte.lastChild);
    
      // Remove all extensions
      while (ttt.hasChildNodes())
        ttt.removeChild(ttt.lastChild);
      
      for (var i = 0; i < numUpdates; i++)
      {
        var item = this._updates.getItemByIndex(i);
        
        // Creates the item row
        var itemRow = document.createElement("row");
        
        // Creates the item image
        var itemImage = document.createElement("image");
        itemImage.setAttribute("class", "un-icons");
        itemImage.setAttribute("icon", item.type);
        itemRow.appendChild(itemImage);
        
        // Creates the item label with new version
        var itemLabel = document.createElement("label");
        itemLabel.setAttribute("class", "bold");
        itemLabel.setAttribute("value", item.name + " " + item.newVersion);
        itemRow.appendChild(itemLabel);
        
        // Creates the item label with current version
        var itemLabel = document.createElement("label");
        itemLabel.setAttribute("value", "(" + UN_getBundleString("tb-tooltip-currently") + " " + item.curVersion + ")");
        itemRow.appendChild(itemLabel);
        
        switch(item.type)
        {
          case "extension":
            tte.appendChild(itemRow);
            break;
          case "theme":
            ttt.appendChild(itemRow);
            break;
        }
      }
      
      // Extensions grid
      tte.parentNode.setAttribute("collapsed", !tte.hasChildNodes());
      tte.parentNode.setAttribute("hidden", !tte.hasChildNodes());
      
      // Themes grid
      ttt.parentNode.setAttribute("collapsed", !ttt.hasChildNodes());
      ttt.parentNode.setAttribute("hidden", !ttt.hasChildNodes());
    }
    
    // Display updates appropriately
    UN_setEBIA("un-toolbar-tooltip-updates", "collapsed", updates);
  },
  
  observe: function(aSubject, aTopic, aData)
  {
    if (aTopic == "nsPref:changed")
    {
      switch (aData)
      {
        case "auto.install":
        {
          // Check for auto install
          if (UN_getBoolPref("auto.install"))
            setTimeout("UN_gOverlay.installUpdates()", 1000);
          
          break;
        }
        case "icon.always-display":
        {
          // Updates the icons
          this._updateIcons();
          break;
        }
        case "statusbar":
        case "statusbar.always-last":
        case "statusbar.position":
        {
          // Checks the statusbar
          this._checkStatusbar();
          break;
        }
      }
    }
    else if (aTopic == UN_NOTIFY_TOPIC)
    {
      // Check notify type
      switch (aData)
      {
        case UN_NOTIFY_TYPE_CHANGE:
        {
          // Check for auto install
          if (UN_getBoolPref("auto.install"))
            setTimeout("UN_gOverlay.installUpdates()", 1000);
          
          break;
        }
        case UN_NOTIFY_TYPE_RESTART:
        {
          // Restart is required
          this._restart = true;
          
          // Check for auto restart
          if (UN_getBoolPref("auto.restart"))
            setTimeout("UN_gOverlay.checkRestart(true)", 1000);
          
          break;
        }
        case UN_NOTIFY_TYPE_BUSY_NONE:
        case UN_NOTIFY_TYPE_BUSY_CHECKING:
        case UN_NOTIFY_TYPE_BUSY_DOWNLOAD:
        case UN_NOTIFY_TYPE_BUSY_INSTALL:
        {
          // Sets the busy status
          this._busyStatus = aData;
          break;
        }
      }
      
      // Updates the icons
      this._updateIcons();
    }
  }
}