// Update Notifier
// By Todd Long <longfocus@gmail.com>
// http://www.longfocus.com/firefox/updatenotifier/

const UN_CC = Components.classes;
const UN_CI = Components.interfaces;

const UN_BRANCH = "longfocus.updatenotifier.";
const UN_PROPERTIES = "chrome://updatenotifier/locale/updatenotifier.properties";
const UN_WEBSITE = "http://www.longfocus.com/firefox/updatenotifier/";

/**
 * Document element
 */
function UN_getEBI(aId) {
  return document.getElementById(aId);
}

function UN_setEBIA(aId, aAttr, aValue) {
  UN_getEBI(aId).setAttribute(aAttr, aValue);
}

/**
 * String bundle
 */
function UN_getBundle() {
  return UN_CC["@mozilla.org/intl/stringbundle;1"]
           .getService(UN_CI.nsIStringBundleService)
           .createBundle(UN_PROPERTIES);
}

function UN_getBundleString(aName) {
  return UN_getBundle().GetStringFromName(aName);
}

function UN_getBundleFString(aName, aParams) {
  return UN_getBundle().formatStringFromName(aName, aParams, aParams.length);
}

/**
 * Prompt service
 */
function UN_confirm(aMsg) {
  return UN_CC["@mozilla.org/embedcomp/prompt-service;1"]
           .getService(UN_CI.nsIPromptService)
           .confirm(window, UN_getBundleString("un-ext-name"), aMsg);
}

/**
 * Branch services
 */
function UN_getBranch() {
  return UN_CC["@mozilla.org/preferences-service;1"]
           .getService(UN_CI.nsIPrefService)
           .getBranch(UN_BRANCH);
}

function UN_getBoolPref(aName) {
  return UN_getBranch().getBoolPref(aName);
}

function UN_setBoolPref(aName, aValue) {
  UN_getBranch().setBoolPref(aName, aValue);
}

function UN_getIntPref(aName) {
  return UN_getBranch().getIntPref(aName);
}

function UN_setIntPref(aName, aValue) {
  UN_getBranch().setIntPref(aName, aValue);
}

/**
 * Misc
 */
function UN_restart() {
  UN_CC["@mozilla.org/toolkit/app-startup;1"]
    .getService(UN_CI.nsIAppStartup)
    .quit(UN_CI.nsIAppStartup.eRestart | UN_CI.nsIAppStartup.eAttemptQuit);
}

function UN_getPlatform() {
  return UN_CC["@mozilla.org/xre/app-info;1"]
           .getService(UN_CI.nsIXULAppInfo).name;
}

function UN_isFirefox() {
  return (UN_getPlatform() == "Firefox");
}

function UN_getAddon(aId) {
  return UN_CC["@mozilla.org/extensions/manager;1"]
           .getService(UN_CI.nsIExtensionManager)
           .getItemForID(aId);
}

function UN_getCurrentTheme()
{
  return UN_CC["@mozilla.org/preferences-service;1"]
           .getService(UN_CI.nsIPrefBranch)
           .getCharPref("general.skins.selectedSkin");
}

function UN_getStatusbarPanels() {
  if (UN_getBrowser())
    return UN_getBrowser().UN_getEBI("status-bar").childNodes.length;
  else
    return null;
}

function UN_getBrowser() {
  return UN_CC["@mozilla.org/appshell/window-mediator;1"]
           .getService(UN_CI.nsIWindowMediator)
           .getMostRecentWindow("navigator:browser");
}

function UN_getMyVersion() {
  return UN_getAddon("{95f24680-9e31-11da-a746-0800200c9a66}").version;
}

function UN_visitSite(aSite) {
  var ios = UN_CC["@mozilla.org/network/io-service;1"].getService(UN_CI.nsIIOService);
  var refString = UN_WEBSITE + "v/" + UN_getMyVersion() + "/";
  var refUri = ios.newURI(refString, null, null);
  
  if (UN_getBrowser())
    UN_getBrowser().document.getElementById("content").addTab(aSite, refUri);
  else {
    try {
      var uri = ios.newURI(aSite, null, null);
      var protocolSvc = UN_CC['@mozilla.org/uriloader/external-protocol-service;1']
                          .getService(UN_CI.nsIExternalProtocolService);
      if (!protocolSvc.isExposedProtocol(uri.scheme)) {
        protocolSvc.loadUrl(uri, refUri);
        return;
      }
    } catch(e) {}
    
    var win = window;
    if (window instanceof UN_CI.nsIDOMChromeWindow) {
      while (win.opener && !win.opener.closed) {
        win = win.opener;
      }
    }
    win.open(href);
  }
}